#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # 1A.algo - Quicksort
# 
# Cet énoncé a pour objectif de présenter l'algorithme de tri [quicksort](https://fr.wikipedia.org/wiki/Tri_rapide) qui permet de trier par ordre croissant un ensemble d'éléments (ici des chaînes de caractères) avec un coût moyen.

# Lorsqu'on parle de coût moyen, cela signifie que le coût n'est pas constant en fonction de la dimension du problème. Ici, le coût moyen désigne le coût moyen d'un tri \textit{quicksort} obtenu en faisant la moyenne du coût du même algorithme sur toutes les permutations possibles de l'ensemble de départ. en $O(n \ln n)$ où $n$ est le nombre d'éléments à classer. Le tri *quicksort* apparaît rarement sous la forme d'un graphe : il est plus simple à programmer sans les graphes mais il est plus simple à appréhender avec les graphes. Dans cette dernière version, l'algorithme insère un à un les éléments d'une liste à trier dans un graphe. Chaque noeud de ce graphe est relié à deux autres noeuds.

# * Un noeud ``avant`` ou ``"<"`` qui permet d'accéder à des éléments classés avant celui de ce noeud.
# * Un noeud ``apres`` ou ``">"`` qui permet d'accéder à des éléments classés après celui de ce noeud.
# 

# Les noeuds ``avant`` et ``apres`` sont appelés les successeurs. Le terme opposé est prédécesseur. Ces deux noeuds ont nécessairement un prédécesseur mais un noeud n'a pas forcément de successeurs. S'il en avait toujours un, l'arbre serait infini.

# In[1]:


from jyquickhelper import add_notebook_menu
add_notebook_menu()


# ## Q1 : classe
# 
# On cherche à construire une classe ayant pour nom ``NoeudTri`` et qui contient une chaîne de caractères initialisée lors de la création de la classe : ``n = NoeudTri("essai")``.

# In[2]:





# ## Q2 : __str__
# 
# On écrit la méthode ``__str__`` de sorte que l'instruction ``print(n)`` affiche la chaîne de caractères que contient ``n``.

# In[3]:





# ## Q3 : avant, après
# 
# On cherche maintenant à définir d'autres noeuds, reliés à des attributs ``avant`` et ``apres``. On suppose que les noeuds utilisent l'attribut ``mot``, on crée alors une méthode ``insere(s)`` qui :
# 
# * Si ``s < self.mot``, alors on ajoute l'attribut ``avant = NoeudTri(s)``.
# * Si ``s > self.mot``, alors on ajoute l'attribut ``apres = NoeudTri(s)``.

# In[4]:





# ## Q4 : __str__
# 
# La méthode ``__str__`` n'affiche pour le moment qu'un mot. Il s'agit maintenant de prendre en compte les attributs ``avant`` et ``apres`` afin que l'instruction ``print(n)`` affiche ``avant.__str__()`` et ``apres.__str__()``. Il faudra également faire en sorte que la méthode ``avant.__str__()`` ne soit appelée que si l'attribut ``avant`` existe. Comme la liste des mots à trier est finie, il faut bien que certains noeuds n'aient pas de successeurs. Qu'est-ce qu'affiche le programme suivant ? 

# In[5]:


racine = NoeudTri("un")  # noeud tri n'est pas encore défini
racine.insere ("unite")
racine.insere ("deux")
print(racine)


# ## Q5 : insere
# 
# Est-il possible de trier plus de trois mots avec ce programme ? Que faut-il modifier dans la méthode ``insere`` afin de pouvoir trier un nombre quelconque de mots ?

# ## Q6 : exception
# 
# Ajouter le code nécessaire afin que la méthode ``insere`` génère une exception lorsqu'un mot déjà présent dans l'arbre est à nouveau inséré.

# In[6]:





# ## Q7 : graph
# 
# On se propose de construire une image représentant l'arbre contenant les mots triés par l'algorithme *quicksort*. Cette représentation utilise le module [blocdiag](http://blockdiag.com/en/) ou on peut utiliser la fonction [draw_diagram](http://www.xavierdupre.fr/app/pyensae/helpsphinx/pyensae/graphhelper/blockdiag_helper.html?highlight=draw_diagram#pyensae.graphhelper.blockdiag_helper.draw_diagram).

# In[7]:


from pyensae.graphhelper import draw_diagram
img = draw_diagram("""
        blockdiag {
            A -> B -> C -> D;
            A -> E -> F;
            F -> G [label = "edge-FG"];
            E [label="label-E"]
        }
        """)
img


# In[8]: